"""
Noms de couleurs particulières qui sont aux coins du cube RGB.
Ainsi que la couleur complètement transparent.
"""

from __future__ import annotations

from pytamaro.color_names import black, blue, cyan, green, magenta, red, transparent, white, yellow
from pytamaro.fr.color import Couleur

noir: Couleur = black
"""Couleur noire

:meta hide-value:
"""

rouge: Couleur = red
"""Couleur rouge

:meta hide-value:
"""

vert: Couleur = green
"""Couleur verte

:meta hide-value:
"""

bleu: Couleur = blue
"""Couleur bleue

:meta hide-value:
"""

jaune: Couleur = yellow
"""Couleur jaune

:meta hide-value:
"""

magenta: Couleur = magenta  # pylint: disable=self-assigning-variable
"""Couleur magenta

:meta hide-value:
"""

cyan: Couleur = cyan
"""Couleur cyan

:meta hide-value:
"""

blanc: Couleur = white
"""Couleur blanche

:meta hide-value:
"""

transparent: Couleur = transparent  # pylint: disable=self-assigning-variable
"""Couleur complètement transparente

:meta hide-value:
"""
